/* This file is part of the KDE project
   Copyright (C) 2002 Simon Hausmann <hausmann@kde.org>

   This library is free software; you can redistribute it and/or
   modify it under the terms of the GNU Library General Public
   License as published by the Free Software Foundation; either
   version 2 of the License, or (at your option) any later version.

   This library is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   Library General Public License for more details.

   You should have received a copy of the GNU Library General Public License
   along with this library; see the file COPYING.LIB.  If not, write to
   the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
   Boston, MA 02111-1307, USA.
*/
#ifndef FRONTEND_H
#define FRONTEND_H

#include <qobject.h>
#include <qdict.h>
#include <qguardedptr.h>

#include "connectioninfo.h"
#include "message.h"

#include "messagedispatcher.h"

namespace IRC
{

class Connection;
class MessageDispatcher;
class Window;
class WindowForwarder;
class Command;
class MessageFilterGroup;
class NamedMessageFilterGroup;
class CTCPHandler;

class Frontend : public QObject
{
    Q_OBJECT
public:
    enum WindowCreationPolicy { CreateIfNotExisting, ReturnDefaultWindow, ReturnNullIfNotExisting };

    Frontend( Connection *connection, const ConnectionInfo &connInfo,
              QObject *parent, const char *name = 0 );
    virtual ~Frontend();

    Window *windowForChannel( const QString &channel, WindowCreationPolicy winPolicy = ReturnDefaultWindow );

    Window *defaultWindow();

    bool hasWindow( const QString &channel ) const;

    ConnectionInfo &connectionInfo() { return m_connInfo; }

    MessageFilterGroup &messageFilter() { return *m_messageFilter; }

    Message createPrefixedMessage( const QString &type = QString::null, const QStringList &parameters = QStringList() );

    Message createPrefixedMessage( const Message &other )
    {
        return createPrefixedMessage( other.type(), other.parameters() );
    }

    template <class MessageImpl>
    Message createPrefixedMessage( const QStringList &parameters = QStringList() )
    {
        return createPrefixedMessage( MessageImpl::name(), parameters );
    }

    Person userJoinedChannel( const QString &nick, Window *window );
    void userLeftChannel( const QString &nick, Window *window );

    void installReplyHandler( const QCString &messageType, QObject *handler, MessageDispatcher::MessageReceiverType receiverType );

    CTCPHandler *ctcpHandler() { return m_ctcpHandler; }

signals:
    void loggedIn();

    void userDetailsAvailable( const IRC::Person &user );

    // ### emitted from the template
    void windowCreated( IRC::Window *window );

public slots:
    virtual void sendOneWayMessage( const IRC::Message &msg );
    virtual void sendMessage( const IRC::Message &msg, QObject *replyHandler );

    void executeCommand( IRC::Command *cmd );

    void unhandledMessage( const IRC::Message &msg );

    void updateUserDetails( const PersonList &users );
    void updateUserDetails( const QString &nick );

protected:
    virtual Window *createWindow( const QString &channel ) = 0;
    virtual Window *createDefaultWindow() = 0;

private slots:
    void windowDestroyed( Window *window );

    void processInput( const QString &input );

    void connected();
    void connectionClosed();

    void handlePingMessage( const IRC::PingMessage &ping );
    void handleQuitMessage( const IRC::QuitMessage &msg );
    void handleNickMessage( const IRC::NickMessage &msg );

    void unregisterCommand( IRC::Command *cmd );

    void nextUserUpdate();
    void nextUserUpdateDelayed();

    void updateUserDetails( const IRC::Person &userDetails );

private:
    void connectCommand( Command *command );
    void connectReplyHandler( const QCString &msgType, QObject *replyHandler,
                              MessageDispatcher::MessageReceiverType receiverType = MessageDispatcher::TemporaryMessageReceiver );
    void connectReplyHandler( QObject *replyHandler, const Message::NumericMessageTypeArray &replyCodes,
                              MessageDispatcher::MessageReceiverType receiverType = MessageDispatcher::TemporaryMessageReceiver );

    void registerInternalMessageTypes();
    void registerStandardMessageFilters();

    void connectWindow( Window *wnd );

    QDict<Window> m_windows;

    QGuardedPtr<Window> m_defaultWindow;

    MessageDispatcher *m_dispatcher;
    WindowForwarder *m_forwarder;

    QGuardedPtr<Connection> m_connection;
    ConnectionInfo m_connInfo;

    MessageFilterGroup *m_messageFilter;
    NamedMessageFilterGroup *m_namedMessageFilters;

    CTCPHandler *m_ctcpHandler;

    PersonList m_users;
    QStringList m_pendingUserUpdates;
    QGuardedPtr<Command> m_userUpdateCommand;
    bool m_userUpdateRunning : 1;
};

template <class WindowImpl>
class GenericFrontend : public Frontend
{
public:
    GenericFrontend( Connection *connection, const ConnectionInfo &connInfo,
                     QObject *parent, const char *name = 0 )
        : Frontend( connection, connInfo, parent, name )
    {}

protected:
    virtual Window *createWindow( const QString &channel )
    {
        Window *wnd = new WindowImpl( channel, this );
        emit windowCreated( wnd );
        return wnd;
    }

    virtual Window *createDefaultWindow( )
    {
        Window *wnd = new WindowImpl( connectionInfo().host(), this );
        emit windowCreated( wnd );
        return wnd;
    }
};

}

#endif // FRONTEND_H

/* vim: et sw=4 ts=4
 */
